@@ -788,7 +788,7 @@ int main (int argc, char **argv) {
 	int opt;
 	int sendflags = 0;
 	char *msgpattern = "librdkafka_performance testing!";
-	int msgsize = (int)strlen(msgpattern);
+	int msgsize = -1;
 	const char *debug = NULL;
 	rd_ts_t now;
 	char errstr[512];
@@ -1248,6 +1248,9 @@ int main (int argc, char **argv) {
 	if (msgcnt != -1)
 		forever = 0;
 
+	if (msgsize == -1)
+		msgsize = (int)strlen(msgpattern);
+
 	topic = topics->elems[0].topic;
 
 	if (mode == 'P') {
@@ -1400,11 +1403,17 @@ int main (int argc, char **argv) {
 			cnt.msgs++;
 			cnt.bytes += msgsize;
 
-                        if (rate_sleep)
-                                do_sleep(rate_sleep);
-
 			/* Must poll to handle delivery reports */
-			rd_kafka_poll(rk, 0);
+			if (rate_sleep) {
+				rd_ts_t next = rd_clock() + (rd_ts_t) rate_sleep;
+				do {
+					rd_kafka_poll(rk,
+						      (int)RD_MAX(0,
+						      (next - rd_clock()) / 1000));
+				} while (next > rd_clock());
+			} else {
+				rd_kafka_poll(rk, 0);
+			}
 
 			print_stats(rk, mode, otype, compression);
 		}
