@@ -26,7 +26,8 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#pragma once
+#ifndef _RDKAFKA_BROKER_H_
+#define _RDKAFKA_BROKER_H_
 
 #include "rdkafka_feature.h"
 
@@ -64,13 +65,18 @@ struct rd_kafka_broker_s { /* rd_kafka_broker_t */
 	TAILQ_HEAD(, rd_kafka_toppar_s) rkb_toppars;
 	int                 rkb_toppar_cnt;
 
-        /* Underflowed toppars that are eligible for fetching. */
-        CIRCLEQ_HEAD(, rd_kafka_toppar_s) rkb_fetch_toppars;
-        int                 rkb_fetch_toppar_cnt;
-        rd_kafka_toppar_t  *rkb_fetch_toppar_next;  /* Next 'first' toppar
-                                                     * in fetch list.
-                                                     * This is used for
-                                                     * round-robin. */
+        /* Active toppars that are eligible for:
+         *  - (consumer) fetching due to underflow
+         *  - (producer) producing
+         *
+         * The circleq provides round-robin scheduling for both cases.
+         */
+        CIRCLEQ_HEAD(, rd_kafka_toppar_s) rkb_active_toppars;
+        int                 rkb_active_toppar_cnt;
+        rd_kafka_toppar_t  *rkb_active_toppar_next;  /* Next 'first' toppar
+                                                      * in fetch list.
+                                                      * This is used for
+                                                      * round-robin. */
 
 
         rd_kafka_cgrp_t    *rkb_cgrp;
@@ -143,6 +149,9 @@ struct rd_kafka_broker_s { /* rd_kafka_broker_t */
 
         int                 rkb_req_timeouts;  /* Current value */
 
+        rd_ts_t             rkb_ts_tx_last;    /**< Timestamp of last
+                                                *   transmitted requested */
+
 	rd_ts_t             rkb_ts_metadata_poll; /* Next metadata poll time */
 	int                 rkb_metadata_fast_poll_cnt; /* Perform fast
 							 * metadata polls. */
@@ -162,6 +171,10 @@ struct rd_kafka_broker_s { /* rd_kafka_broker_t */
 	rd_kafka_bufq_t     rkb_retrybufs;
 
 	rd_avg_t            rkb_avg_int_latency;/* Current internal latency period*/
+        rd_avg_t            rkb_avg_outbuf_latency; /**< Current latency
+                                                     *   between buf_enq0
+                                                     *   and writing to socket
+                                                     */
 	rd_avg_t            rkb_avg_rtt;        /* Current RTT period */
 	rd_avg_t            rkb_avg_throttle;   /* Current throttle period */
 
@@ -268,6 +281,16 @@ rd_kafka_broker_t *rd_kafka_broker_any_usable (rd_kafka_t *rk, int timeout_ms,
 
 rd_kafka_broker_t *rd_kafka_broker_prefer (rd_kafka_t *rk, int32_t broker_id, int state);
 
+rd_kafka_broker_t *
+rd_kafka_broker_get_async (rd_kafka_t *rk, int32_t broker_id, int state,
+                           rd_kafka_enq_once_t *eonce);
+
+rd_kafka_broker_t *rd_kafka_broker_controller (rd_kafka_t *rk, int state,
+                                               rd_ts_t abs_timeout);
+rd_kafka_broker_t *
+rd_kafka_broker_controller_async (rd_kafka_t *rk, int state,
+                                  rd_kafka_enq_once_t *eonce);
+
 int rd_kafka_brokers_add0 (rd_kafka_t *rk, const char *brokerlist);
 void rd_kafka_broker_set_state (rd_kafka_broker_t *rkb, int state);
 
@@ -275,6 +298,10 @@ void rd_kafka_broker_fail (rd_kafka_broker_t *rkb,
 			   int level, rd_kafka_resp_err_t err,
 			   const char *fmt, ...);
 
+void rd_kafka_broker_conn_closed (rd_kafka_broker_t *rkb,
+                                  rd_kafka_resp_err_t err,
+                                  const char *errstr);
+
 void rd_kafka_broker_destroy_final (rd_kafka_broker_t *rkb);
 
 #define rd_kafka_broker_destroy(rkb)                                    \
@@ -325,4 +352,34 @@ void rd_kafka_broker_wakeup (rd_kafka_broker_t *rkb);
 int rd_kafka_brokers_get_state_version (rd_kafka_t *rk);
 int rd_kafka_brokers_wait_state_change (rd_kafka_t *rk, int stored_version,
 					int timeout_ms);
+int rd_kafka_brokers_wait_state_change_async (rd_kafka_t *rk,
+                                              int stored_version,
+                                              rd_kafka_enq_once_t *eonce);
 void rd_kafka_brokers_broadcast_state_change (rd_kafka_t *rk);
+
+
+
+/**
+ * Updates the current toppar active round-robin next pointer.
+ */
+static RD_INLINE RD_UNUSED
+void rd_kafka_broker_active_toppar_next (rd_kafka_broker_t *rkb,
+                                        rd_kafka_toppar_t *sugg_next) {
+        if (CIRCLEQ_EMPTY(&rkb->rkb_active_toppars) ||
+            (void *)sugg_next == CIRCLEQ_ENDC(&rkb->rkb_active_toppars))
+                rkb->rkb_active_toppar_next = NULL;
+        else if (sugg_next)
+                rkb->rkb_active_toppar_next = sugg_next;
+        else
+                rkb->rkb_active_toppar_next =
+                        CIRCLEQ_FIRST(&rkb->rkb_active_toppars);
+}
+
+
+void rd_kafka_broker_active_toppar_add (rd_kafka_broker_t *rkb,
+                                        rd_kafka_toppar_t *rktp);
+
+void rd_kafka_broker_active_toppar_del (rd_kafka_broker_t *rkb,
+                                        rd_kafka_toppar_t *rktp);
+
+#endif /* _RDKAFKA_BROKER_H_ */
