@@ -32,6 +32,7 @@
 #include "rdkafka_msgset.h"
 #include "rdkafka_topic.h"
 #include "rdkafka_partition.h"
+#include "rdkafka_header.h"
 #include "rdkafka_lz4.h"
 
 #include "snappy.h"
@@ -47,8 +48,11 @@ typedef struct rd_kafka_msgset_writer_s {
         int     msetw_features;          /* Protocol features to use */
         int     msetw_msgcntmax;         /* Max number of messages to send
                                           * in a batch. */
-        size_t  msetw_messages_len;      /* Total size of Messages, without
+        size_t  msetw_messages_len;      /* Total size of Messages, with Message
+                                          * framing but without
                                           * MessageSet header */
+        size_t  msetw_messages_kvlen;    /* Total size of Message keys
+                                          * and values */
 
         size_t  msetw_MessageSetSize;    /* Current MessageSetSize value */
         size_t  msetw_of_MessageSetSize; /* offset of MessageSetSize */
@@ -364,11 +368,13 @@ rd_kafka_msgset_writer_write_Produce_header (rd_kafka_msgset_writer_t *msetw) {
  *
  * @remark This currently constructs the entire ProduceRequest, containing
  *         a single outer MessageSet for a single partition.
+ *
+ * @locality broker thread
  */
 static int rd_kafka_msgset_writer_init (rd_kafka_msgset_writer_t *msetw,
                                          rd_kafka_broker_t *rkb,
                                          rd_kafka_toppar_t *rktp) {
-        int msgcnt = rd_atomic32_get(&rktp->rktp_xmit_msgq.rkmq_msg_cnt);
+        int msgcnt = rktp->rktp_xmit_msgq.rkmq_msg_cnt;
 
         if (msgcnt == 0)
                 return 0;
@@ -433,15 +439,52 @@ rd_kafka_msgset_writer_write_msg_payload (rd_kafka_msgset_writer_t *msetw,
          * room in the buffer we'll copy the payload to the buffer,
          * otherwise we push a reference to the memory. */
         if (rkm->rkm_len <= (size_t)rk->rk_conf.msg_copy_max_size &&
-            rd_buf_write_remains(&rkbuf->rkbuf_buf) > rkm->rkm_len)
+            rd_buf_write_remains(&rkbuf->rkbuf_buf) > rkm->rkm_len) {
                 rd_kafka_buf_write(rkbuf,
                                    rkm->rkm_payload, rkm->rkm_len);
-        else
+                if (free_cb)
+                        free_cb(rkm->rkm_payload);
+        } else
                 rd_kafka_buf_push(rkbuf, rkm->rkm_payload, rkm->rkm_len,
                                   free_cb);
 }
 
 
+/**
+ * @brief Write message headers to buffer.
+ *
+ * @remark The enveloping HeaderCount varint must already have been written.
+ * @returns the number of bytes written to msetw->msetw_rkbuf
+ */
+static size_t
+rd_kafka_msgset_writer_write_msg_headers (rd_kafka_msgset_writer_t *msetw,
+                                          const rd_kafka_headers_t *hdrs) {
+        rd_kafka_buf_t *rkbuf = msetw->msetw_rkbuf;
+        const rd_kafka_header_t *hdr;
+        int i;
+        size_t start_pos = rd_buf_write_pos(&rkbuf->rkbuf_buf);
+        size_t written;
+
+        RD_LIST_FOREACH(hdr, &hdrs->rkhdrs_list, i) {
+                rd_kafka_buf_write_varint(rkbuf, hdr->rkhdr_name_size);
+                rd_kafka_buf_write(rkbuf,
+                                   hdr->rkhdr_name, hdr->rkhdr_name_size);
+                rd_kafka_buf_write_varint(rkbuf,
+                                          hdr->rkhdr_value ?
+                                          (int64_t)hdr->rkhdr_value_size : -1);
+                rd_kafka_buf_write(rkbuf,
+                                   hdr->rkhdr_value,
+                                   hdr->rkhdr_value_size);
+        }
+
+        written = rd_buf_write_pos(&rkbuf->rkbuf_buf) - start_pos;
+        rd_dassert(written == hdrs->rkhdrs_ser_size);
+
+        return written;
+}
+
+
+
 /**
  * @brief Write message to messageset buffer with MsgVersion 0 or 1.
  * @returns the number of bytes written.
@@ -536,6 +579,13 @@ rd_kafka_msgset_writer_write_msg_v2 (rd_kafka_msgset_writer_t *msetw,
         size_t sz_KeyLen;
         size_t sz_ValueLen;
         size_t sz_HeaderCount;
+        int    HeaderCount = 0;
+        size_t HeaderSize = 0;
+
+        if (rkm->rkm_headers) {
+                HeaderCount = rkm->rkm_headers->rkhdrs_list.rl_cnt;
+                HeaderSize  = rkm->rkm_headers->rkhdrs_ser_size;
+        }
 
         /* All varints, except for Length, needs to be pre-built
          * so that the Length field can be set correctly and thus have
@@ -555,7 +605,8 @@ rd_kafka_msgset_writer_write_msg_v2 (rd_kafka_msgset_writer_t *msetw,
                 rkm->rkm_payload ? (int32_t)rkm->rkm_len :
                 (int32_t)RD_KAFKAP_BYTES_LEN_NULL);
         sz_HeaderCount = rd_uvarint_enc_i32(
-                varint_HeaderCount, sizeof(varint_HeaderCount), 0);
+                varint_HeaderCount, sizeof(varint_HeaderCount),
+                (int32_t)HeaderCount);
 
         /* Calculate MessageSize without length of Length (added later)
          * to store it in Length. */
@@ -567,7 +618,8 @@ rd_kafka_msgset_writer_write_msg_v2 (rd_kafka_msgset_writer_t *msetw,
                 rkm->rkm_key_len +
                 sz_ValueLen +
                 rkm->rkm_len +
-                sz_HeaderCount;
+                sz_HeaderCount +
+                HeaderSize;
 
         /* Length */
         sz_Length = rd_uvarint_enc_i64(varint_Length, sizeof(varint_Length),
@@ -599,9 +651,14 @@ rd_kafka_msgset_writer_write_msg_v2 (rd_kafka_msgset_writer_t *msetw,
         if (rkm->rkm_payload)
                 rd_kafka_msgset_writer_write_msg_payload(msetw, rkm, free_cb);
 
-        /* HeaderCount (headers currently not implemented) */
+        /* HeaderCount */
         rd_kafka_buf_write(rkbuf, varint_HeaderCount, sz_HeaderCount);
 
+        /* Headers array */
+        if (rkm->rkm_headers)
+                rd_kafka_msgset_writer_write_msg_headers(msetw,
+                                                         rkm->rkm_headers);
+
         /* Return written message size */
         return MessageSize;
 }
@@ -649,6 +706,8 @@ rd_kafka_msgset_writer_write_msg (rd_kafka_msgset_writer_t *msetw,
 /**
  * @brief Write as many messages from the given message queue to
  *        the messageset.
+ *
+ *        May not write any messages.
  */
 static void
 rd_kafka_msgset_writer_write_msgq (rd_kafka_msgset_writer_t *msetw,
@@ -663,10 +722,11 @@ rd_kafka_msgset_writer_write_msgq (rd_kafka_msgset_writer_t *msetw,
         rd_ts_t MaxTimestamp = 0;
         rd_kafka_msg_t *rkm;
         int msgcnt = 0;
+        const rd_ts_t now = rd_clock();
 
         /* Internal latency calculation base.
          * Uses rkm_ts_timeout which is enqueue time + timeout */
-        int_latency_base = rd_clock() +
+        int_latency_base = now +
                 (rktp->rktp_rkt->rkt_conf.message_timeout_ms * 1000);
 
         /* Acquire BaseTimestamp from first message. */
@@ -690,10 +750,18 @@ rd_kafka_msgset_writer_write_msgq (rd_kafka_msgset_writer_t *msetw,
                         break;
                 }
 
+                if (unlikely(rkm->rkm_u.producer.ts_backoff > now)) {
+                        /* Stop accumulation when we've reached
+                         * a message with a retry backoff in the future */
+                        break;
+                }
+
                 /* Move message to buffer's queue */
                 rd_kafka_msgq_deq(rkmq, rkm, 1);
                 rd_kafka_msgq_enq(&rkbuf->rkbuf_msgq, rkm);
 
+                msetw->msetw_messages_kvlen += rkm->rkm_len + rkm->rkm_key_len;
+
                 /* Add internal latency metrics */
                 rd_avg_add(&rkb->rkb_avg_int_latency,
                            int_latency_base - rkm->rkm_ts_timeout);
@@ -731,9 +799,11 @@ rd_kafka_msgset_writer_compress_gzip (rd_kafka_msgset_writer_t *msetw,
         const void *p;
         size_t rlen;
         int r;
-
+        int comp_level =
+                msetw->msetw_rktp->rktp_rkt->rkt_conf.compression_level;
+				
         memset(&strm, 0, sizeof(strm));
-        r = deflateInit2(&strm, Z_DEFAULT_COMPRESSION,
+        r = deflateInit2(&strm, comp_level,
                          Z_DEFLATED, 15+16,
                          8, Z_DEFAULT_STRATEGY);
         if (r != Z_OK) {
@@ -868,9 +938,12 @@ static int
 rd_kafka_msgset_writer_compress_lz4 (rd_kafka_msgset_writer_t *msetw,
                                      rd_slice_t *slice, struct iovec *ciov) {
         rd_kafka_resp_err_t err;
+        int comp_level =
+                msetw->msetw_rktp->rktp_rkt->rkt_conf.compression_level;
         err = rd_kafka_lz4_compress(msetw->msetw_rkb,
                                     /* Correct or incorrect HC */
                                     msetw->msetw_MsgVersion >= 1 ? 1 : 0,
+                                    comp_level,
                                     slice, &ciov->iov_base, &ciov->iov_len);
         return (err ? -1 : 0);
 }
@@ -1108,6 +1181,9 @@ rd_kafka_msgset_writer_finalize (rd_kafka_msgset_writer_t *msetw,
         rd_assert(len > 0);
         rd_assert(len <= (size_t)rktp->rktp_rkt->rkt_rk->rk_conf.max_msg_size);
 
+        rd_atomic64_add(&rktp->rktp_c.tx_msgs, cnt);
+        rd_atomic64_add(&rktp->rktp_c.tx_msg_bytes, msetw->msetw_messages_kvlen);
+
         /* Compress the message set */
         if (rktp->rktp_rkt->rkt_conf.compression_codec)
                 rd_kafka_msgset_writer_compress(msetw, &len);
@@ -1128,7 +1204,6 @@ rd_kafka_msgset_writer_finalize (rd_kafka_msgset_writer_t *msetw,
                    cnt, msetw->msetw_MessageSetSize,
                    msetw->msetw_ApiVersion, msetw->msetw_MsgVersion);
 
-
         return rkbuf;
 }
 
@@ -1144,6 +1219,8 @@ rd_kafka_msgset_writer_finalize (rd_kafka_msgset_writer_t *msetw,
  *
  * @returns the buffer to transmit or NULL if there were no messages
  *          in messageset.
+ *
+ * @locality broker thread
  */
 rd_kafka_buf_t *
 rd_kafka_msgset_create_ProduceRequest (rd_kafka_broker_t *rkb,
