@@ -26,7 +26,8 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#pragma once
+#ifndef _RDKAFKA_PROTO_H_
+#define _RDKAFKA_PROTO_H_
 
 
 #include "rdendian.h"
@@ -81,7 +82,16 @@ struct rd_kafkap_reqhdr {
 #define RD_KAFKAP_DeleteAcls    31
 #define RD_KAFKAP_DescribeConfigs 32
 #define RD_KAFKAP_AlterConfigs  33
-#define RD_KAFKAP__NUM          34
+#define RD_KAFKAP_AlterReplicaLogDirs 34
+#define RD_KAFKAP_DescribeLogDirs 35
+#define RD_KAFKAP_SaslAuthenticate 36
+#define RD_KAFKAP_CreatePartitions 37
+#define RD_KAFKAP_CreateDelegationToken 38
+#define RD_KAFKAP_RenewDelegationToken 39
+#define RD_KAFKAP_ExpireDelegationToken 40
+#define RD_KAFKAP_DescribeDelegationToken 41
+#define RD_KAFKAP_DeleteGroups 42
+#define RD_KAFKAP__NUM         43
         int16_t  ApiVersion;
         int32_t  CorrId;
         /* ClientId follows */
@@ -134,11 +144,22 @@ const char *rd_kafka_ApiKey2str (int16_t ApiKey) {
                 [RD_KAFKAP_CreateAcls] = "CreateAcls",
                 [RD_KAFKAP_DeleteAcls] = "DeleteAcls",
                 [RD_KAFKAP_DescribeConfigs] = "DescribeConfigs",
-                [RD_KAFKAP_AlterConfigs] = "AlterConfigs"
+                [RD_KAFKAP_AlterConfigs] = "AlterConfigs",
+                [RD_KAFKAP_AlterReplicaLogDirs] = "AlterReplicaLogDirs",
+                [RD_KAFKAP_DescribeLogDirs] = "DescribeLogDirs",
+                [RD_KAFKAP_SaslAuthenticate] = "SaslAuthenticate",
+                [RD_KAFKAP_CreatePartitions] = "CreatePartitions",
+                [RD_KAFKAP_CreateDelegationToken] = "CreateDelegationToken",
+                [RD_KAFKAP_RenewDelegationToken] = "RenewDelegationToken",
+                [RD_KAFKAP_ExpireDelegationToken] = "ExpireDelegationToken",
+                [RD_KAFKAP_DescribeDelegationToken] = "DescribeDelegationToken",
+                [RD_KAFKAP_DeleteGroups] = "DeleteGroups"
+
         };
         static RD_TLS char ret[32];
 
-        if (ApiKey < 0 || ApiKey >= (int)RD_ARRAYSIZE(names)) {
+        if (ApiKey < 0 || ApiKey >= (int)RD_ARRAYSIZE(names) ||
+            !names[ApiKey]) {
                 rd_snprintf(ret, sizeof(ret), "Unknown-%hd?", ApiKey);
                 return ret;
         }
@@ -220,6 +241,8 @@ typedef struct rd_kafkap_str_s {
 /* strndup() a Kafka string */
 #define RD_KAFKAP_STR_DUP(kstr) rd_strndup((kstr)->str, RD_KAFKAP_STR_LEN(kstr))
 
+#define RD_KAFKAP_STR_INITIALIZER { .len = RD_KAFKAP_STR_LEN_NULL, .str = NULL }
+
 /**
  * Frees a Kafka string previously allocated with `rd_kafkap_str_new()`
  */
@@ -496,3 +519,5 @@ typedef struct rd_kafka_buf_s rd_kafka_buf_t;
 #define RD_KAFKAP_MSGSET_V2_OF_BaseTimestamp    (8+4+4+1+4+2+4)
 #define RD_KAFKAP_MSGSET_V2_OF_MaxTimestamp     (8+4+4+1+4+2+4+8)
 #define RD_KAFKAP_MSGSET_V2_OF_RecordCount      (8+4+4+1+4+2+4+8+8+8+2+4)
+
+#endif /* _RDKAFKA_PROTO_H_ */
