@@ -26,7 +26,8 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#pragma once
+#ifndef _RDKAFKA_TIMER_H_
+#define _RDKAFKA_TIMER_H_
 
 #include "rd.h"
 
@@ -49,6 +50,7 @@ typedef struct rd_kafka_timer_s {
 
 	rd_ts_t rtmr_next;
 	rd_ts_t rtmr_interval;   /* interval in microseconds */
+        rd_bool_t rtmr_oneshot;  /**< Only fire once. */
 
 	void  (*rtmr_callback) (rd_kafka_timers_t *rkts, void *arg);
 	void   *rtmr_arg;
@@ -56,13 +58,18 @@ typedef struct rd_kafka_timer_s {
 
 
 
-void rd_kafka_timer_stop (rd_kafka_timers_t *rkts,
-                          rd_kafka_timer_t *rtmr, int lock);
-void rd_kafka_timer_start (rd_kafka_timers_t *rkts,
-			   rd_kafka_timer_t *rtmr, rd_ts_t interval,
-			   void (*callback) (rd_kafka_timers_t *rkts,
-                                             void *arg),
-			   void *arg);
+int rd_kafka_timer_stop (rd_kafka_timers_t *rkts,
+                         rd_kafka_timer_t *rtmr, int lock);
+void rd_kafka_timer_start0 (rd_kafka_timers_t *rkts,
+                            rd_kafka_timer_t *rtmr, rd_ts_t interval,
+                            rd_bool_t oneshot,
+                            void (*callback) (rd_kafka_timers_t *rkts,
+                                              void *arg),
+                            void *arg);
+#define rd_kafka_timer_start(rkts,rtmr,interval,callback,arg) \
+        rd_kafka_timer_start0(rkts,rtmr,interval,rd_false,callback,arg)
+#define rd_kafka_timer_start_oneshot(rkts,rtmr,interval,callback,arg)   \
+        rd_kafka_timer_start0(rkts,rtmr,interval,rd_true,callback,arg)
 
 void rd_kafka_timer_backoff (rd_kafka_timers_t *rkts,
 			     rd_kafka_timer_t *rtmr, int backoff_us);
@@ -75,3 +82,5 @@ rd_ts_t rd_kafka_timers_next (rd_kafka_timers_t *rkts, int timeout_ms,
 void rd_kafka_timers_run (rd_kafka_timers_t *rkts, int timeout_us);
 void rd_kafka_timers_destroy (rd_kafka_timers_t *rkts);
 void rd_kafka_timers_init (rd_kafka_timers_t *rkte, rd_kafka_t *rk);
+
+#endif /* _RDKAFKA_TIMER_H_ */
