@@ -78,7 +78,7 @@ int main_0040_io_event (int argc, char **argv) {
 
 	rk_p = test_create_producer();
 	rkt_p = test_create_producer_topic(rk_p, topic, NULL);
-	err = test_auto_create_topic_rkt(rk_p, rkt_p);
+	err = test_auto_create_topic_rkt(rk_p, rkt_p, tmout_multip(5000));
         TEST_ASSERT(!err, "Topic auto creation failed: %s",
                     rd_kafka_err2str(err));
 
